# -*- coding: UTF-8 -*-

from bohrium_open_sdk.opensdk._resource import SyncAPIResource
from bohrium_open_sdk.opensdk._response import APIResponse


class AppDB(SyncAPIResource):

    def __init__(self, _client) -> None:
        super().__init__(_client)

    def create(self, data):
        response = self._client.post(
            "/openapi/v1/db/app/data/insert", json=data
        )
        return APIResponse(response).json

    def delete(self, data):
        response = self._client.delete(
            "/openapi/v1/db/app/data", json=data
        )
        return APIResponse(response).json

    def update(self, data):
        response = self._client.post(
            "/openapi/v1/db/app/data/update", json=data
        )
        return APIResponse(response).json

    def query(self, data):
        response = self._client.post(
            "/openapi/v1/db/app/data/list", json=data
        )
        return APIResponse(response).json

    def count(self, data):
        response = self._client.post(
            "/openapi/v1/db/app/data/count", json=data
        )
        return APIResponse(response).json

    def create_table(self, data):
        response = self._client.post(
            "/openapi/v1/db/app/table/create", json=data
        )
        return APIResponse(response).json

    def delete_table(self, data):
        response = self._client.delete(
            "/openapi/v1/db/app/table", json=data
        )
        return APIResponse(response).json




# -*- coding: UTF-8 -*-

from enum import Enum
from typing import List

'''
{
    "type":1,
    "field": "1",
    "op": "eq",
    "value": 2,
    "condition":"and/or",
    "sub":[]
}
'''


class Op(Enum):
    EQ = "eq"
    NEQ = "neq"
    BETWEEN = "between"
    GT = "gt"
    GTE = "gte"
    LT = "lt"
    LTE = "lte"
    LIKE = "like"


class Condition(Enum):
    AND = "and"
    OR = "or"


class NodeType(Enum):
    NORMAL = 1
    Operator = 2
    UNKNOWN = 3


class Node:
    def __init__(self, key: str, op: Op, value):
        self.key = key
        self.op = op
        self.value = value
        self.type = NodeType.UNKNOWN

    def Or(self, *args):
        pass

    def And(self, *args):
        pass

    def get_op(self) -> Op:
        return self.op

    def dict(self):
        pass


class OperatorNode(Node):
    def __init__(self, op: Op, condition: Condition):
        super().__init__("", op, None)
        self.type = NodeType.Operator
        self.condition = condition
        self.stack = []
        self.children = []
        self.has_or = False

    def Or(self, *args):
        if len(args) == 1:
            args[0].condition = Condition.OR
            self._add_or(args[0])
            return self
        elif len(args) == 3:
            node = Entity(args[0], args[1], args[2])
            self._add_or(node)
            return self
        else:
            raise ValueError("args count err")

    def _add_or(self, node):  # 弹栈、打包、压入
        self.has_or = True
        if len(self.stack) > 0:
            group_list = []
            for child in self.stack:
                group_list.append(child)
            self.stack = []
            if len(group_list) > 0:
                self.children.append(group_list)

        self.stack.append(node)

    def And(self, *args):
        if len(args) == 1:
            args[0].condition = Condition.AND
            self._add_and(args[0])
            return self
        elif len(args) == 3:
            node = Entity(args[0], args[1], args[2])
            self._add_and(node)
            return self
        else:
            raise ValueError("args count err")

    def _add_and(self, node):
        self.stack.append(node)

    def dict(self):
        if len(self.stack) > 0:
            group_list = []
            for child in self.stack:
                group_list.append(child)
            self.stack = []
            if len(group_list) > 0:
                self.children.append(group_list)

        res = {}
        if self.has_or:
            res["type"] = NodeType.Operator.value
            res["groupOperator"] = Condition.OR.value
        else:
            res["type"] = NodeType.NORMAL.value
            res["groupOperator"] = Condition.AND.value

        sub = []
        for child in self.children:
            if self.has_or:
                if len(child) == 0:
                    continue
                elif len(child) == 1:
                    for c in child:
                        if isinstance(c, (OperatorNode, Entity)):
                            sub.append(c.dict())
                        else:
                            raise ValueError("can not find this node")
                else:
                    and_group = {
                        "type": NodeType.Operator.value,
                        "groupOperator": Condition.AND.value,
                    }
                    child_sub = []
                    for c in child:
                        if isinstance(c, (OperatorNode, Entity)):
                            child_sub.append(c.dict())
                        else:
                            raise ValueError("can not find this node")
                    and_group["sub"] = child_sub
                    sub.append(and_group)

            else:
                if len(child) == 0:
                    continue
                else:
                    for c in child:
                        if isinstance(c, (OperatorNode, Entity)):
                            sub.append(c.dict())
                        else:
                            raise ValueError("can not find this node")

        if len(sub) == 0:
            return None

        res = {
            "type": self.type.value if self.type else None,
            "groupOperator": "or" if self.has_or else "and",
            "sub": sub,
        }

        res = {k: v for k, v in res.items() if v is not None}
        return res


class Entity(Node):
    def __init__(self, key: str, op: Op, value):
        super().__init__(key, op, value)
        self.type = NodeType.NORMAL

    def Or(self, key: str, op: Op, value, condition: Condition):
        raise ValueError("entity can not add or")

    def And(self, key: str, op: Op, value, conditin: Condition):
        raise ValueError("entity can not add and")

    def dict(self):
        res = {
            "type": self.type.value,
            "field": self.key,
            "operator": self.op.value if self.op else None,
            "value": self.value,
        }

        res = {k: v for k, v in res.items() if v is not None}
        return res



from bohrium_open_sdk.opensdk.resources.app.app_job import AppJob
from bohrium_open_sdk.opensdk._resource import SyncAPIResource
from bohrium_open_sdk.opensdk._response import APIResponse


class App(SyncAPIResource):
    job: AppJob

    def __init__(self, _client) -> None:
        self.job = AppJob(_client)
        super().__init__(_client)

    def get_app_info(self, app_key: str):
        response = self._client.get(
            "/openapi/v1/square/app/schema", params={"appKey": app_key}
        )
        return APIResponse(response).json



# -*- coding: UTF-8 -*-

from bohrium_open_sdk import OpenSDK


class App:
    def __init__(self, base_url="https://openapi.test.dp.tech", access_key="", timeout=60):
        # timeout 默认 1min
        self.client = OpenSDK(base_url=base_url, access_key=access_key, timeout=timeout)

    def get_user_info(self):
        resp = self.client.user.get_info()
        data = resp.get("data")
        return data.get("user_id"), data.get("org_id")

    def get_app_id(self, app_key):
        resp = self.client.app.get_app_info(app_key)
        return resp.get("data").get("id")

    def create(self, data):
        return self.client.app_db.create(data)

    def delete(self, data):
        return self.client.app_db.delete(data)

    def count(self, data):
        return self.client.app_db.count(data)

    def update(self, data):
        return self.client.app_db.update(data)

    def query(self, data):
        return self.client.app_db.query(data)

    def create_table(self, data):
        return self.client.app_db.create_table(data)

    def delete_table(self, data):
        return self.client.app_db.delete_table(data)



from typing import List


class JobAddRequest(object):
    def __init__(
        self,
        job_type: str = "",
        oss_path: List[str] = [],
        job_group_id: int = 0,
        project_id: int = 0,
        rerun: int = 0,
        job_name: str = "",
        image_name: str = "",
        disk_size: int = 0,
        scass_type: str = "",
        nnode: int = 0,
        cmd: str = "",
        log_files: List[str] = [],
        out_files: List[str] = [],
        platform: str = "",
        region: str = "",
        zone: str = "",
        on_demand: int = 0,
        checkpoint_time: int = 0,
        checkpoint_files: List[str] = [],
        cli_instance_id: str = "",
        download_path: str = "",
        work_path: str = "",
        job_id: int = 0,
        max_run_time: int = 0,
        client: str = "",
        bohr_job_group_id: int = 0,
        max_reschedule_times: int = 0,
        dataset_path: List[str] = [],
        local_download_path: str = "",
        input_directory: str = "",
        instance_group_id: int = 0,
        input_file_type: int = 2,
        input_file_method: int = 4,
    ):
        self.jobType = job_type
        self.ossPath = oss_path
        self.jobGroupId = job_group_id
        self.projectId = project_id
        self.rerun = rerun
        self.jobName = job_name
        self.imageName = image_name
        self.diskSize = disk_size
        self.scassType = scass_type
        self.nnode = nnode
        self.instanceGroupId = instance_group_id
        self.cmd = cmd
        self.logFiles = log_files
        self.outFiles = out_files
        self.platform = platform
        self.region = region
        self.zone = zone
        self.onDemand = on_demand
        self.checkpointTime = checkpoint_time
        self.checkpointFiles = checkpoint_files
        self.cliInstanceId = cli_instance_id
        self.downloadPath = download_path
        self.workPath = work_path
        self.jobId = job_id
        self.maxRunTime = max_run_time
        self.client = client
        self.bohrJobGroupId = bohr_job_group_id
        self.inputFileType = input_file_type
        self.inputFileMethod = input_file_method
        self.maxRescheduleTimes = max_reschedule_times
        self.inputDirectory = input_directory
        self.datasetPath = dataset_path
        self.localDownloadPath = local_download_path

    def to_dict(self):
        return {
            "jobType": self.jobType,
            "ossPath": [self.ossPath],
            "jobGroupId": self.jobGroupId,
            "projectId": self.projectId,
            "rerun": self.rerun,
            "jobName": self.jobName,
            "imageName": self.imageName,
            "diskSize": self.diskSize,
            "scassType": self.scassType,
            "nnode": self.nnode,
            "instanceGroupId": self.instanceGroupId,
            "cmd": self.cmd,
            "logFiles": self.logFiles,
            "outFiles": self.outFiles,
            "platform": self.platform,
            "region": self.region,
            "zone": self.zone,
            "onDemand": self.onDemand,
            "checkpointTime": self.checkpointTime,
            "checkpointFiles": self.checkpointFiles,
            "cliInstanceId": self.cliInstanceId,
            "downloadPath": self.downloadPath,
            "workPath": self.workPath,
            "jobId": self.jobId,
            "maxRunTime": self.maxRunTime,
            "client": self.client,
            "bohrJobGroupId": self.bohrJobGroupId,
            "inputFileType": self.inputFileType,
            "inputFileMethod": self.inputFileMethod,
            "maxRescheduleTimes": self.maxRescheduleTimes,
            "inputDirectory": self.inputDirectory,
            "datasetPath": self.datasetPath,
            "localDownloadPath": self.localDownloadPath,
        }


class JobDetailRequest:
    def __init__(
        self,
        id: int,
        bohr_id: int,
        project_id: int,
        user_id: int,
        job_group_id: int,
        job_group_name: str,
        thirdparty_id: int,
        platform: str,
        spend_time: int,
        status: int,
        web_status: int,
        result: str,
        result_url: str,
        cost: str,
        on_demand: int,
        input_data: str,
        error_info: str,
        create_time: str,
        update_time: str,
        job_name: str,
        project_name: str,
        user_name: str,
        end_time: str,
        image_name: str,
        max_run_time: int,
        max_run_time_unit: str,
        machine_type: str,
        region: str,
        work_path: str,
        cmd: str,
        job_files: List[str],
        machines: List[str],
        is_k8s: int,
        enable_web_shell: int,
        web_shell_tag: str,
        download_path: str,
        out_file: List[str],
        auto_download_info: dict,
        client: int,
        metrics: List[dict],
        is_show_metrics: bool,
        running_start_at: int,
        running_end_at: int,
        metrics_tag: str,
    ):
        self.id = id
        self.bohr_id = bohr_id
        self.project_id = project_id
        self.user_id = user_id
        self.job_group_id = job_group_id
        self.job_group_name = job_group_name
        self.thirdparty_id = thirdparty_id
        self.platform = platform
        self.spend_time = spend_time
        self.status = status
        self.web_status = web_status
        self.result = result
        self.result_url = result_url
        self.cost = cost
        self.on_demand = on_demand
        self.input_data = input_data
        self.error_info = error_info
        self.create_time = create_time
        self.update_time = update_time
        self.job_name = job_name
        self.project_name = project_name
        self.user_name = user_name
        self.end_time = end_time
        self.image_name = image_name
        self.max_run_time = max_run_time
        self.max_run_time_unit = max_run_time_unit



import logging
import os
import uuid
from pathlib import Path
from typing import Optional, Union
from unittest import result

from bohrium_open_sdk.opensdk._exceptions import NotFoundError
from bohrium_open_sdk.opensdk._resource import SyncAPIResource
from bohrium_open_sdk.opensdk._response import APIResponse
from bohrium_open_sdk.opensdk._tiefblue_client import Tiefblue
from bohrium_open_sdk.opensdk.types.job.job import JobAddRequest
from bohrium_open_sdk.opensdk._base_client import APIResponseManager


logger = logging.getLogger(__name__)


class Job(SyncAPIResource):

    def detail(self, job_id):
        with APIResponseManager(self._client.get) as api:
            uri = f"/{self._client.api_prefix}/v1/job/{job_id}"
            response = api.get_response(uri)
            return APIResponse(response).json

    def list(
        self,
        *,
        page: int = 1,
        page_size: int = 20,
        query_type: str = "all",
        star: Optional[int] = None,
        version: Optional[int] = 2,
        job_group_search_key: Optional[int] = None,
        group_id: Optional[int] = None,
    ):
        with APIResponseManager(self._client.get) as api:
            # submit_user_id = os.getenv("USER_ID")
            # if not submit_user_id:
                # raise ValueError("user id is required")
            uri = f"/{self._client.api_prefix}/v1/job/list"
            params = {
                "page": page,
                "pageSize": page_size,
                "queryType": query_type,
                # "submitUserId": submit_user_id,
            }
            star and params.update({"star": star})
            version and params.update({"version": version})
            job_group_search_key and params.update({"jobGroupSearchKey": job_group_search_key})
            group_id and params.update({"groupId": group_id})

            response = api.get_response(uri, params=params)
            return APIResponse(response).json
        
        
    def submit(
        self,
        *,
        project_id: int,
        job_name: str,
        machine_type: str,
        cmd: str,
        image_address: str,
        job_group_id: int = 0,
        work_dir: Union[str, Path] = "",
        dataset_path: list = [],
        log_files: list = [],
        out_files: list = [],
    ):
        data = self.create_job(project_id, job_name, job_group_id)
        if not data:
            raise NotFoundError("Create job failed")
        if work_dir != "":
            work_dir = Path(work_dir)
            if not work_dir.exists():
                raise FileNotFoundError
            if work_dir.is_dir():
                self.upload_dir(work_dir, data.get("storePath"), data.get("token"))
            else:
                file_name = work_dir.name
                object_key = data.get("storePath") / file_name
                self.upload(work_dir, object_key, data.get("token"))

        # FIXME: download_path
        # ep = os.path.expanduser(result)
        # p = Path(ep).absolute().resolve()
        # p = p.joinpath(str(uuid.uuid4()) + "_temp.zip")

        job_add_request = JobAddRequest(
            # download_path=str(p.absolute().resolve()),
            dataset_path=dataset_path,
            job_name=job_name,
            project_id=project_id,
            job_id=data["jobId"],
            oss_path=data["storePath"],
            image_name=image_address,
            scass_type=machine_type,
            cmd=cmd,
            log_files=log_files,
            out_files=out_files,
        )
        return self.insert(job_add_request.to_dict())

    def insert(self, data):
        with APIResponseManager(self._client.post) as api:
            uri = f"/{self._client.api_prefix}/v2/job/add"
            response = api.get_response(uri, json=data)
            return APIResponse(response).json


    def delete(self, job_id):
        with APIResponseManager(self._client.post) as api:
            uri = f"/{self._client.api_prefix}/v1/job/del/{job_id}"
            response = api.get_response(uri)
            return APIResponse(response).json

    def terminate(self, job_id):
        with APIResponseManager(self._client.post) as api:
            uri = f"/{self._client.api_prefix}/v1/job/terminate/{job_id}"
            response = api.get_response(uri)
            return APIResponse(response).json

    def kill(self, job_id):
        with APIResponseManager(self._client.post) as api:
            uri = f"/{self._client.api_prefix}/v1/job/kill/{job_id}"
            response = api.get_response(uri)
            return APIResponse(response).json

    def log(self, job_id, log_file="STDOUTERR", page=-1, page_size=8192):
        with APIResponseManager(self._client.get) as api:
            uri = f"/{self._client.api_prefix}/v1/job/{job_id}/log"
            response = api.get_response(
                uri,
                params={"logFile": log_file, "page": page, "pageSize": page_size},
            )
            return APIResponse(response).json

    def create_job(
        self,
        project_id: int,
        name: Optional[str] = None,
        group_id: Optional[int] = 0,
    ):
        with APIResponseManager(self._client.post) as api:
            uri = f"/{self._client.api_prefix}/v1/job/create"
            data = {
                "projectId": project_id,
                "name": name,
                "bohrGroupId": group_id,
            }
            response = api.get_response(uri, json=data)
            return APIResponse(response).data

    def create_job_group(self, project_id, job_group_name):
        with APIResponseManager(self._client.post) as api:
            uri = f"/{self._client.api_prefix}/v1/job_group/add"
            response = api.get_response(
                uri,
                json={"name": job_group_name, "projectId": project_id},
            )

            return APIResponse(response).json

    def upload(
        self,
        file_path: str,
        object_key: str,
        token: str,
    ):
        tiefblue = Tiefblue()
        custom_headers = {"Authorization": f"Bearer {token}"}

        tiefblue.upload_from_file_multi_part(
            object_key=object_key,
            custom_headers=custom_headers,
            file_path=file_path,
            progress_bar=True,
        )

    def upload_dir(self, work_dir: Union[str, Path], store_path: str, token: str):
        if not store_path:
            raise ValueError("store_path is required")
        if not token:
            raise ValueError("token is required")
        work_dir = str(work_dir)
        # TODO: upload rule
        if not work_dir.endswith("/"):
            work_dir = work_dir + "/"
        for root, _, files in os.walk(work_dir):
            for file in files:
                full_path = os.path.join(root, file)
                object_key = full_path.replace(work_dir, store_path)
                self.upload(full_path, object_key, token)

    def download(self, job_id: int, save_path: Union[str, Path]):
        save_path = Path(save_path)
        detail = self.detail(job_id)

        result_url = detail.get("data", {}).get("resultUrl")

        if not result_url:
            raise NotFoundError("Result url not found")

        if not save_path.parent.exists():
            save_path.parent.mkdir(parents=True, exist_ok=True)

        logger.debug(
            f"Downloading job {job_id} to {save_path}, result url: {result_url}"
        )

        tiefblue = Tiefblue()
        tiefblue.download_from_url(result_url, save_path)
        return True



import logging

from bohrium_open_sdk.opensdk._resource import SyncAPIResource
from bohrium_open_sdk.opensdk._response import APIResponse


log = logging.getLogger(__name__)


class User(SyncAPIResource):

    def get_info(self):
        response = self._client.get("openapi/v1/ak/user")
        return APIResponse(response).json
